/** 
 * @file LLSidepanelInventory.h
 * @brief Side Bar "Inventory" panel
 *
 * $LicenseInfo:firstyear=2009&license=viewerlgpl$
 * Second Life Viewer Source Code
 * Copyright (C) 2010, Linden Research, Inc.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License only.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * Linden Research, Inc., 945 Battery Street, San Francisco, CA  94111  USA
 * $/LicenseInfo$
 */

#ifndef LL_LLSIDEPANELINVENTORY_H
#define LL_LLSIDEPANELINVENTORY_H

#include "llpanel.h"

class LLButton;
class LLFolderViewItem;
class LLInboxAddedObserver;
class LLInventoryCategoriesObserver;
class LLInventoryItem;
class LLInventoryPanel;
class LLLayoutPanel;
class LLPanelMainInventory;

class LLSidepanelInventory final : public LLPanel
{
public:
	LLSidepanelInventory();
	virtual ~LLSidepanelInventory();

private:
	void updateInbox();
	
public:
	void observeInboxCreation();
	void observeInboxModifications(const LLUUID& inboxID);

	/*virtual*/ BOOL postBuild() override;
	/*virtual*/ void onOpen(const LLSD& key) override;

	LLInventoryPanel* getActivePanel() const; // Returns an active inventory panel, if any.
	void selectAllItemsPanel();
	LLInventoryPanel* getInboxPanel() const { return mInventoryPanelInbox.get(); }

	LLPanelMainInventory* getMainInventoryPanel() const { return mPanelMainInventory; }
	BOOL isMainInventoryPanelActive() const;

	void clearSelections(bool clearMain, bool clearInbox);
    std::set<LLFolderViewItem*> getInboxSelectionList() const;

	// checks can share selected item(s)
	bool canShare();
	void onToggleInboxBtn();
	void enableInbox(bool enabled);
	void openInbox();
	bool isInboxEnabled() const { return mInboxEnabled; }
	void updateVerbs();

protected:
	void onSelectionChange(const std::deque<LLFolderViewItem*> &items, BOOL user_action);
	void onInboxChanged(const LLUUID& inbox_id);

	//
	// UI Elements
	//
private:
	LLPanel*					mInventoryPanel; // Main inventory view
	LLHandle<LLInventoryPanel>	mInventoryPanelInbox;
	LLPanelMainInventory*		mPanelMainInventory;

protected:
	void 						onBackButtonClicked();

private:
	bool						mInboxEnabled;

	LLInventoryCategoriesObserver* 	mCategoriesObserver;
	LLInboxAddedObserver*			mInboxAddedObserver;
};

#endif //LL_LLSIDEPANELINVENTORY_H
